@@ -42,10 +42,12 @@ allowed_updates = ['callback_query', 'channel_post', 'chat_member', 'chosen_inli
                    'chosen_inline_result', 'edited_channel_post', 'edited_message', 'inline_query',
                    'message', 'my_chat_member', 'poll_answer', 'pre_checkout_query', 'shipping_query']
 
-patterns = {'retry': r'Retry in (\d+) seconds|"Too Many Requests: retry after (\d+)"',
-            'major': 'The (read|write) operation timed out|Backend Error|is currently unavailable.',
-            'minor': 'Failed to establish a new connection|Read timed out.'
-                     '|ServerDisconnectedError|Message_id_invalid|Connection aborted',
+patterns = {'major': 'The (read|write) operation timed out|Backend Error'
+                     '|is currently unavailable.|returned "Internal Error"',
+            'retry': r'Retry in (\d+) seconds|Please try again in (\d+) seconds.|'
+                     r'"Too Many Requests: retry after (\d+)"',
+            'minor': 'Failed to establish a new connection|Read timed out.|ServerDisconnectedError'
+                     '|Message_id_invalid|Connection aborted|Connection reset by peer',
             'block': 'initiate conversation with a user|user is deactivated|Have no rights'
                      '|The group has been migrated|bot was kicked from the supergroup chat'
                      '|bot was blocked by the user|Chat not found|bot was kicked from the group chat'}
@@ -93,15 +95,12 @@ def html_secure(text, reverse=None):
     return text
 
 
-def stamper(date, delta=0, pattern=None):
-    try:
-        if pattern is None:
-            stamp = int(datetime.fromisoformat(str(date)).timestamp())
-        else:
-            stamp = int(calendar.timegm(time.strptime(str(date), pattern)))
-    except IndexError and Exception:
-        stamp = None
-    return stamp - delta * 60 * 60 if stamp else None
+def concurrent_functions(array):
+    array = [array] if type(array) != list else array
+    with concurrent.futures.ThreadPoolExecutor(max_workers=10) as future_executor:
+        futures = [future_executor.submit(future) for future in array]
+        for future in concurrent.futures.as_completed(futures):
+            print(future.result())
 
 
 def chunks(array, separate):
@@ -113,12 +112,15 @@ def chunks(array, separate):
     return separated
 
 
-def concurrent_functions(array):
-    array = [array] if type(array) != list else array
-    with concurrent.futures.ThreadPoolExecutor(max_workers=10) as future_executor:
-        futures = [future_executor.submit(future) for future in array]
-        for future in concurrent.futures.as_completed(futures):
-            print(future.result())
+def stamper(date, delta=0, pattern=None):
+    try:
+        if pattern is None:
+            stamp = int(datetime.fromisoformat(str(date)).timestamp())
+        else:
+            stamp = int(calendar.timegm(time.strptime(str(date), pattern)))
+    except IndexError and Exception:
+        stamp = None
+    return stamp - delta * 60 * 60 if stamp else None
 
 
 def environmental_files(python=None):
@@ -372,13 +374,10 @@ class AuthCentre:
                 message = kwargs['message']
                 response = await task(target_id, message['chat']['id'], message['message_id'])
 
-            elif task_name == 'send_photo' and kwargs.get('path'):
-                response = await task(target_id, types.InputFile(kwargs['path']),
-                                      caption=kwargs.get('caption'), parse_mode='HTML')
-
             elif task_name in ['send_audio', 'send_photo', 'send_video', 'send_voice', 'send_document']:
-                response = await task(target_id, kwargs['file_id'],
-                                      caption=kwargs.get('caption'), parse_mode='HTML')
+                caption = kwargs['text'] if kwargs.get('text') else kwargs.get('caption')
+                file = types.InputFile(kwargs['path']) if kwargs.get('path') else kwargs['file_id']
+                response = await task(target_id, file, caption=caption, parse_mode='HTML')
 
             elif task_name == 'send_message':
                 if kwargs.get('message'):
@@ -395,6 +394,10 @@ class AuthCentre:
                     message['date'] = time_now(self.delta)
                     message['from'] = kwargs['call']['from']
                     log_text = 'Нажал' if log_text is None else log_text
+                    try:
+                        await self.async_bot.answer_callback_query(kwargs['call']['id'], text=kwargs.get('answer'))
+                    except IndexError and Exception:
+                        pass
 
                     if kwargs.get('text'):
                         modified = html_secure(re.sub('<.*?>', '', kwargs['text']), reverse=True).strip()
@@ -412,8 +415,8 @@ class AuthCentre:
                         error_text = ''
                         log_text += ', но получил #timeout'
                         try:
-                            task = self.async_bot.answer_callback_query
-                            await task(kwargs['call']['id'], text=kwargs.get('timeout'))
+                            answer_task = self.async_bot.answer_callback_query
+                            await answer_task(kwargs['call']['id'], text=kwargs.get('timeout'))
                         except IndexError and Exception as answer_error:
                             error_text += f'\nAnswer error: {answer_error}'
                         if re.search('Query is too old', str(error)) is None:
@@ -434,17 +437,16 @@ class AuthCentre:
                     log_text += bold(' [Не отправлено]')
             else:
                 error_text = ''
-                keys = deepcopy(kwargs)
-                for key in keys:
-                    if key in ['call', 'message', 'keyboard'] and keys[key] is not None:
-                        keys[key] = keys[key].to_python()
+                for key in kwargs:
+                    if key in ['call', 'message', 'keyboard'] and kwargs[key] is not None:
+                        kwargs[key] = kwargs[key].to_python()
 
                 if kwargs.get('text'):
                     error_text = f"\nlen(re.sub(<.*?>, text)) = " \
                                  f"{len(re.sub('<.*?>', '', str(kwargs['text'])))}" \
                                  f"\nlen(text) = {len(str(kwargs['text']))}\ntext = {kwargs['text']}"
                 self.dev.executive(f'Not delivered {task_name.upper()} to: {target_id}\n'
-                                   f'Short error: {error}{error_text}\nKeys: {keys}')
+                                   f'Short error: {error}{error_text}\nKeys: {kwargs}')
 
         if log_text is not None:
             message = kwargs['call']['message'] if kwargs.get('call') else kwargs['message']
@@ -501,6 +503,8 @@ class AuthCentre:
                 self.delay = kwargs['LOG_DELAY']
             elif type(kwargs.get('LOG_DELAY')) == str:
                 self.delay = int(re.sub(r'\D', '', kwargs['LOG_DELAY']))
+            elif len(self.chat_ids) == 0:
+                self.delay = 0
             else:
                 self.delay = 15
 
@@ -539,7 +543,7 @@ class AuthCentre:
                     search_start = re.search(r'От: (.*)', chat.description)
                     end_time = search_end.group(1) if search_end else None
                     start_time = search_start.group(1) if search_start else None
-                for sign in ['dev', 'media', 'forward', 'dump']:
+                for sign in ['dev', 'forward', 'media', 'dump']:
                     signs.append(sign) if sign in str(chat.title).lower() else None
                 value.update({
                     'id': chat.id,
@@ -592,6 +596,29 @@ class AuthCentre:
                 head += f'\n{space}👤 {head_text}'
             return head, name, username, space, update
 
+        def reboot(self, dispatcher):
+            def heroku(delay, connection):
+                time.sleep(4)
+                dispatcher.stop_polling()
+                time.sleep(delay+1)
+                for app in connection.apps():
+                    for dyno in app.dynos():
+                        dyno.restart()
+
+            if os.environ.get('api'):
+                if self.delay+5 in [21, 31, 41]:
+                    postfix = 'секунду'
+                elif self.delay+5 in [22, 23, 24, 32, 33, 34]:
+                    postfix = 'секунды'
+                else:
+                    postfix = 'секунд'
+                connect = heroku3.from_key(os.environ['api'])
+                _thread.start_new_thread(heroku, (self.delay, connect,))
+                text, log_text = f'✅ Перезапуск через {self.delay+5} {postfix}.', '[Успешно]'
+            else:
+                text, log_text = '❌ Переменная окружения не установлена.', '[Неудачно]'
+            return bold(text), f' {bold(log_text)}'
+
         def text(self):
             def links(array, title_one='', title_many='', sep='\n'):
                 text, count, enumerated = '', 0, []
@@ -815,6 +842,9 @@ class AuthCentre:
 
                                 elif status['new'] == 'restricted':
                                     action['text'] = 'Ограничил %s'
+                                    if member['username'] == self.username \
+                                            and message['new_chat_member']['can_send_messages'] is False:
+                                        update['reaction'] = '🔕'
 
                                 else:
                                     action['text'] = 'Забрал роль админа у %s'
@@ -826,6 +856,9 @@ class AuthCentre:
                 member_text, _, _ = self.header(member.to_python())
                 emoji = '🤖' if action['tag_type'] == 'bot' else '👤'
                 action['member'] = f"\n{space}{' ' * 5}{emoji} {member_text[:-1]}"
+                if user and user['reaction'] == '🔕' and member['username'] == self.username \
+                        and message['new_chat_member']['can_send_messages'] is True:
+                    update['reaction'] = '♿'
             else:
                 chat_type = 'канал' if message['chat']['type'] == 'channel' else 'чат'
                 if status['old'] in ['left', 'kicked']:
@@ -840,7 +873,7 @@ class AuthCentre:
             text += action['member'] if action.get('member') else ''
             return text, update if update and user else None, greeting
 
-        async def data(self, message, user, hard=True):
+        async def data(self, message, user=None, hard=True):
             text = ''
             head, name, username, space, update = self.head(message, user)
 
